# -*- coding: utf-8 -*-
"""The package 'epyunit' extends the standard PyUnit package for blackbox tests on arbitrary executables.

"""
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

__package__ = 'epyunit'
__all__=[
    "SystemCalls",
    "SubprocessUnit",
    "SProcUnitRules",
    "selftest",
    "checkRDbg",
    "PyDevERDbg",
    "TestExecutable",
]
